import { ResultSet } from "./Api";

export class Transaction {
  private _queries: Array<TransactionQuery> = [];

  push(
    query: string,
    params: Record<string, string | number | TransactionQueryRef>
  ): TransactionQuery {
    const stackedQuery = new TransactionQuery(
      query,
      params,
      this._queries.length
    );
    this._queries.push(stackedQuery);
    return stackedQuery;
  }

  get queries(): ReadonlyArray<TransactionQuery> {
    return this._queries;
  }
}

export class TransactionResult {
  constructor(private resultSets: ReadonlyArray<ResultSet>) {}

  resolve(ref: TransactionQueryRef) {
    return this.resultSets[ref.queryIndex][ref.row][ref.column];
  }
}

export interface TransactionQueryRef {
  queryIndex: number;
  row: number;
  column: number;
}

export class TransactionQuery {
  constructor(
    public readonly query: string,
    public readonly params: Record<
      string,
      string | number | TransactionQueryRef
    >,
    public readonly index: number
  ) {}

  ref(row = 0, column = 0): TransactionQueryRef {
    return {
      queryIndex: this.index,
      row,
      column,
    };
  }
}

export function isTransactionQueryRef(
  value: string | number | TransactionQueryRef
): value is TransactionQueryRef {
  return (
    typeof value === "object" &&
    ["queryIndex", "row", "column"].every((key) =>
      Object.keys(value).includes(key)
    )
  );
}

export interface TransactionBackend {
  execute(transaction: Transaction): Promise<TransactionResult>;
}

export class NativeTransactionBackend {
  async execute(): Promise<TransactionResult> {
    throw new Error("NativeTransactionBackend.execute not yet implemented.");
  }
}
